/* USER CODE BEGIN Header */
/**
  ******************************************************************************
  * @file           : main.c
  * @brief          : Main program body
  ******************************************************************************
  ** This notice applies to any and all portions of this file
  * that are not between comment pairs USER CODE BEGIN and
  * USER CODE END. Other portions of this file, whether 
  * inserted by the user or by software development tools
  * are owned by their respective copyright owners.
  *
  * COPYRIGHT(c) 2023 STMicroelectronics
  *
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met:
  *   1. Redistributions of source code must retain the above copyright notice,
  *      this list of conditions and the following disclaimer.
  *   2. Redistributions in binary form must reproduce the above copyright notice,
  *      this list of conditions and the following disclaimer in the documentation
  *      and/or other materials provided with the distribution.
  *   3. Neither the name of STMicroelectronics nor the names of its contributors
  *      may be used to endorse or promote products derived from this software
  *      without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
  * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
  ******************************************************************************
  */
/* USER CODE END Header */

/* Includes ------------------------------------------------------------------*/
#include "main.h"

/* Private includes ----------------------------------------------------------*/
/* USER CODE BEGIN Includes */
#include <stdbool.h>
#include <string.h>
#include <math.h>
/* USER CODE END Includes */

/* Private typedef -----------------------------------------------------------*/
/* USER CODE BEGIN PTD */

/* USER CODE END PTD */

/* Private define ------------------------------------------------------------*/
/* USER CODE BEGIN PD */

/* USER CODE END PD */

/* Private macro -------------------------------------------------------------*/
/* USER CODE BEGIN PM */

/* USER CODE END PM */

/* Private variables ---------------------------------------------------------*/
ADC_HandleTypeDef hadc1;

I2C_HandleTypeDef hi2c1;

TIM_HandleTypeDef htim1;

/* USER CODE BEGIN PV */
/* Private variables ---------------------------------------------------------*/

/* USER CODE END PV */

/* Private function prototypes -----------------------------------------------*/
void SystemClock_Config(void);
static void MX_GPIO_Init(void);
static void MX_I2C1_Init(void);
static void MX_ADC1_Init(void);
static void MX_TIM1_Init(void);
/* USER CODE BEGIN PFP */
/* Private function prototypes -----------------------------------------------*/

/* USER CODE END PFP */

/* Private user code ---------------------------------------------------------*/
/* USER CODE BEGIN 0 */

const uint8_t eeprom_i2c_addr = (0x50 << 1);
#define EEPROM_PAGE_SIZE 32
#define KEYER_MESSAGE_EEPROM_ADDR       0x0000

typedef enum {
    CW_SEND_NONE = 0,
    CW_SEND_DIT = 1,
    CW_SEND_DAH = 2,
} CwSend_t;

CwSend_t lastSent = CW_SEND_NONE;
CwSend_t nextSend = CW_SEND_NONE;

// ditTimeMs = 60*1000/(50*WPM)
// where 50 is the length in dits of "PARIS "
// see https://morsecode.world/international/timing.html
uint32_t ditTimeMs = 60;
uint32_t speedWPM = 20;
bool straightKey = false;

bool allowTransmit = true;

#define MAX_KEYER_MESSAGE_LENGTH 500

typedef struct {
    uint16_t checksum;
    uint16_t length;
    char message[MAX_KEYER_MESSAGE_LENGTH];
} KeyerMessage_t;

KeyerMessage_t keyerMessage = {
    .checksum = 0,
    .length = 0,
};

#define BUTTON_DEBOUNCE_TIME_MS 200
typedef enum {
    BUTTON_STATUS_PRESSED = 0,
    BUTTON_STATUS_RELEASED = 1,
    BUTTON_STATUS_DEBOUNCE = 2,
} ButtonStatus_t;

typedef enum {
    SEG_G = (1<<0),
    SEG_F = (1<<1),
    SEG_E = (1<<2),
    SEG_D = (1<<3),
    SEG_C = (1<<4),
    SEG_B = (1<<5),
    SEG_A = (1<<6),
    SEG_DP = (1<<7),
} SEGMENTS;

void updateKeyerSpeed(bool display);

// http://en.wikipedia.org/wiki/Jenkins_hash_function
uint32_t jenkinsHash(const uint8_t *data, const size_t len) {
  uint32_t hash, i;
  for(hash = i = 0; i < len; ++i) {
    hash += data[i];
    hash += (hash << 10);
    hash ^= (hash >> 6);
  }
  hash += (hash << 3);
  hash ^= (hash >> 11);
  hash += (hash << 15);
  return hash;
}

void loadKeyerMessage() {
    HAL_StatusTypeDef status;
    uint16_t savedChecksum;
    KeyerMessage_t savedKeyerMessage;

    for(;;) {
        status = HAL_I2C_IsDeviceReady(&hi2c1, eeprom_i2c_addr, 3, HAL_MAX_DELAY);
        if(status == HAL_OK)
            break;
    }

    HAL_I2C_Mem_Read(&hi2c1, eeprom_i2c_addr, KEYER_MESSAGE_EEPROM_ADDR, I2C_MEMADD_SIZE_16BIT,
            (uint8_t*)&savedKeyerMessage, sizeof(savedKeyerMessage), HAL_MAX_DELAY);

    savedChecksum = savedKeyerMessage.checksum;
    savedKeyerMessage.checksum = 0;

    if((jenkinsHash((const uint8_t*)&savedKeyerMessage, sizeof(savedKeyerMessage)) & 0xFFFF) == savedChecksum) {
        keyerMessage = savedKeyerMessage;
    } else {
        // checksum error, do nothing
    }
}

void saveKeyerMessage() {
    HAL_StatusTypeDef status;
    uint16_t checksum;
    size_t bytes_to_write = sizeof(keyerMessage);
    uint8_t* struct_offset = (uint8_t*)&keyerMessage;
    uint16_t eeprom_addr = KEYER_MESSAGE_EEPROM_ADDR;

    keyerMessage.checksum = 0;
    checksum = (jenkinsHash((const uint8_t*)&keyerMessage, sizeof(keyerMessage)) & 0xFFFF);
    keyerMessage.checksum = checksum;

    // sizeof(keyerMessage) exceeds EEPROM_PAGE_SIZE
    // so we have to write it in batches
    while(bytes_to_write > 0) {
        size_t batch_size = bytes_to_write;
        if(batch_size > EEPROM_PAGE_SIZE) {
            batch_size = EEPROM_PAGE_SIZE;
        }

        for(;;) {
            status = HAL_I2C_IsDeviceReady(&hi2c1, eeprom_i2c_addr, 3, HAL_MAX_DELAY);
            if(status == HAL_OK)
                break;
        }

        HAL_I2C_Mem_Write(&hi2c1, eeprom_i2c_addr, eeprom_addr, I2C_MEMADD_SIZE_16BIT,
            struct_offset, batch_size, HAL_MAX_DELAY);

        bytes_to_write -= batch_size;
        struct_offset += batch_size;
        eeprom_addr += batch_size;
    }
}

uint8_t charToSegments(char c) {
    switch(c) {
        case ' ':
            return 0;
        case '-':
            return SEG_G;
        case '0':
            return SEG_A | SEG_B | SEG_C | SEG_D | SEG_E | SEG_F;
        case '1':
            return SEG_B | SEG_C;
        case '2':
            return SEG_A | SEG_B | SEG_D | SEG_E | SEG_G;
        case '3':
            return SEG_A | SEG_B | SEG_C | SEG_D | SEG_G;
        case '4':
            return SEG_B | SEG_C | SEG_F | SEG_G;
        case 'S':
        case '5':
            return SEG_A | SEG_C | SEG_D | SEG_F | SEG_G;
        case '6':
            return SEG_A | SEG_C | SEG_D | SEG_E | SEG_F | SEG_G;
        case '7':
            return SEG_A | SEG_B | SEG_C;
        case '8':
            return SEG_A | SEG_B | SEG_C | SEG_D | SEG_E | SEG_F | SEG_G;
        case '9':
            return SEG_A | SEG_B | SEG_C | SEG_D | SEG_F | SEG_G;
        case 'P':
            return SEG_A | SEG_B | SEG_E | SEG_F | SEG_G;
        case 'R':
            return SEG_E | SEG_G;
        default:
            return SEG_A | SEG_B | SEG_E | SEG_G; // question mark
    }
}

void displayDig1(char c) {
    uint8_t segments = charToSegments(c);

    HAL_GPIO_WritePin(GPIOB, GPIO_PIN_11,
        segments & SEG_A ? GPIO_PIN_SET : GPIO_PIN_RESET);

    HAL_GPIO_WritePin(GPIOB, GPIO_PIN_10,
        segments & SEG_B ? GPIO_PIN_SET : GPIO_PIN_RESET);

    HAL_GPIO_WritePin(GPIOB, GPIO_PIN_1,
        segments & SEG_C ? GPIO_PIN_SET : GPIO_PIN_RESET);

    HAL_GPIO_WritePin(GPIOB, GPIO_PIN_0,
        segments & SEG_D ? GPIO_PIN_SET : GPIO_PIN_RESET);

    HAL_GPIO_WritePin(GPIOA, GPIO_PIN_7,
        segments & SEG_E ? GPIO_PIN_SET : GPIO_PIN_RESET);

    HAL_GPIO_WritePin(GPIOA, GPIO_PIN_6,
        segments & SEG_F ? GPIO_PIN_SET : GPIO_PIN_RESET);

    HAL_GPIO_WritePin(GPIOA, GPIO_PIN_5,
        segments & SEG_G ? GPIO_PIN_SET : GPIO_PIN_RESET);
}

void displayDig2(char c) {
    uint8_t segments = charToSegments(c);

    HAL_GPIO_WritePin(GPIOA, GPIO_PIN_3,
        segments & SEG_A ? GPIO_PIN_SET : GPIO_PIN_RESET);

    HAL_GPIO_WritePin(GPIOA, GPIO_PIN_2,
        segments & SEG_B ? GPIO_PIN_SET : GPIO_PIN_RESET);

    HAL_GPIO_WritePin(GPIOA, GPIO_PIN_1,
        segments & SEG_C ? GPIO_PIN_SET : GPIO_PIN_RESET);

    HAL_GPIO_WritePin(GPIOA, GPIO_PIN_0,
        segments & SEG_D ? GPIO_PIN_SET : GPIO_PIN_RESET);

    HAL_GPIO_WritePin(GPIOC, GPIO_PIN_15,
        segments & SEG_E ? GPIO_PIN_SET : GPIO_PIN_RESET);

    HAL_GPIO_WritePin(GPIOC, GPIO_PIN_14,
        segments & SEG_F ? GPIO_PIN_SET : GPIO_PIN_RESET);

    HAL_GPIO_WritePin(GPIOC, GPIO_PIN_13,
        segments & SEG_G ? GPIO_PIN_SET : GPIO_PIN_RESET);
}

void keyDown() {
    __HAL_TIM_SET_COMPARE(&htim1, TIM_CHANNEL_1, 128); // tone on

    if(allowTransmit) { // playback only otherwise
        HAL_GPIO_WritePin(GPIOB, GPIO_PIN_12, GPIO_PIN_SET);
    }
}

void keyUp() {
    __HAL_TIM_SET_COMPARE(&htim1, TIM_CHANNEL_1, 0); // tone off

    if(allowTransmit) { // playback only otherwise
        HAL_GPIO_WritePin(GPIOB, GPIO_PIN_12, GPIO_PIN_RESET);
    }
}

bool buttonDitPressed() {
    return (HAL_GPIO_ReadPin(GPIOB, GPIO_PIN_13) == GPIO_PIN_RESET);
}

bool buttonDahPressed() {
    return (HAL_GPIO_ReadPin(GPIOB, GPIO_PIN_14) == GPIO_PIN_RESET);
}

ButtonStatus_t buttonPressed(GPIO_TypeDef *GPIOx, uint16_t GPIO_Pin, uint32_t* lastPressed) {
    if(HAL_GPIO_ReadPin(GPIOx, GPIO_Pin) == GPIO_PIN_RESET) {
        uint32_t now = HAL_GetTick();
        if(now - *lastPressed > BUTTON_DEBOUNCE_TIME_MS) {
            *lastPressed = now;
            return BUTTON_STATUS_PRESSED;
        } else {
            return BUTTON_STATUS_DEBOUNCE;
        }
    }
    return BUTTON_STATUS_RELEASED;
}

bool buttonSendPressed() {
    static uint32_t lastPressed = 0;
    return (buttonPressed(GPIOA, GPIO_PIN_11, &lastPressed) == BUTTON_STATUS_PRESSED);
}

bool buttonPlayPressed() {
    static uint32_t lastPressed = 0;
    return (buttonPressed(GPIOA, GPIO_PIN_12, &lastPressed) == BUTTON_STATUS_PRESSED);
}

bool buttonRecPressed() {
    static uint32_t lastPressed = 0;
    return (buttonPressed(GPIOA, GPIO_PIN_15, &lastPressed) == BUTTON_STATUS_PRESSED);
}

bool anyButtonPressed() {
    return buttonDitPressed() ||
        buttonDahPressed() ||
        buttonSendPressed() ||
        buttonPlayPressed() ||
        buttonRecPressed();
}

void processStraightKeyerLogic(bool pressed) {
    if(pressed) {
        keyDown();
    } else  {
        keyUp();
    }
}

static size_t iambicKeyerLogOffset = 0;
static char iambicKeyerLog[MAX_KEYER_MESSAGE_LENGTH] = {0};

void iambicKeyerLogReset() {
    iambicKeyerLogOffset = 0;
}

size_t iambicKeyerLogSize() {
    return iambicKeyerLogOffset;
}

size_t iambicKeyerLogSpaceLeft() {
    return MAX_KEYER_MESSAGE_LENGTH - iambicKeyerLogOffset;
}

// Iambic keyer logs up to MAX_KEYER_MESSAGE_LENGTH symbols.
// '.' is a dit
// '-' is a dah
// ' ' is a pause between letters
// '/' is a pause between words
// Multiple pauses between letters are logged as one pause.
// Multiple pauses between words are also logged as one pause.
// Pauses in the beginning of the message are not logged.
void iambicKeyerLogEmit(char c) {
    if(iambicKeyerLogSpaceLeft() == 0) {
        // truncate the message if there is not space left in the buffer
        return;
    }

    if(((c == ' ') || (c == '/')) && (iambicKeyerLogOffset == 0)) {
        // don't log the pauses in the beginning of the message
        return;
    }

    if((c == ' ') && iambicKeyerLog[iambicKeyerLogOffset-1] == ' ') {
        // multiple pauses between letters are logged as one pause
        return;
    }

    if((c == '/') && iambicKeyerLog[iambicKeyerLogOffset-1] == '/') {
        // multiple pauses between words are logged as one pause.
        return;
    }

    if((c == '/') && iambicKeyerLog[iambicKeyerLogOffset-1] == ' ') {
        // previously logged pause between letters became a pause between words
        iambicKeyerLog[iambicKeyerLogOffset-1] = '/';
        return;
    }

    if((c == ' ') && iambicKeyerLog[iambicKeyerLogOffset-1] == '/') {
        // to my knowledge this should never happen...
        return;
    }

    // log the symbol
    iambicKeyerLog[iambicKeyerLogOffset] = c;
    iambicKeyerLogOffset++;
}

void processIambicKeyerLogic(bool ditPressed, bool dahPressed) {
    static int32_t sendFinish = 0;
    static int32_t keyReadTimeout = 0;
    static int32_t sendTimeout = 0;
    static bool isSending = false;

    int32_t now = HAL_GetTick();

    if(isSending && (now >= sendFinish)) {
        keyUp();
        isSending = false;
    } else if(!isSending && (now - sendFinish) > ditTimeMs*5) {
        // multiple pauses between words will be logged as one pause
        // pauses in the beginning of the message will be ignored
        iambicKeyerLogEmit('/');
    } else if(!isSending && (now - sendFinish) > ditTimeMs*2) {
        // multiple pauses between letters will be logged as one pause
        // pauses in the beginning of the message will be ignored
        iambicKeyerLogEmit(' ');
    }

    if(nextSend == CW_SEND_NONE) {
        if(ditPressed && (lastSent == CW_SEND_DAH)) {
            nextSend = CW_SEND_DIT;
        } else if(dahPressed && (lastSent == CW_SEND_DIT)) {
            nextSend = CW_SEND_DAH;
        } else if(now > keyReadTimeout) {
            if(ditPressed) {
                nextSend = CW_SEND_DIT;
            } else if (dahPressed) {
                nextSend = CW_SEND_DAH;
            }
        }
    }

    if((now > sendTimeout) && (nextSend != CW_SEND_NONE)) {
        if(nextSend == CW_SEND_DIT) {
            iambicKeyerLogEmit('.');
            sendFinish = now + ditTimeMs;
            keyReadTimeout = now + ditTimeMs*2;
        } else if(nextSend == CW_SEND_DAH) {
            iambicKeyerLogEmit('-');
            sendFinish = now + ditTimeMs*3;
            keyReadTimeout = now + ditTimeMs*3;
        }

        lastSent = nextSend;
        nextSend = CW_SEND_NONE;
        sendTimeout = sendFinish + ditTimeMs;
        isSending = true;
        keyDown();
    }
}

void playbackSavedMessage() {
    int32_t sendStart = 0;
    int32_t sendFinish = 0;
    bool isSending = false;
    int32_t now;
    size_t symbolsLeft = keyerMessage.length;
    size_t currentSymbolIdx = 0;
    char c;

    while(symbolsLeft > 0) {
        now = HAL_GetTick();

        if((!isSending) && (now >= sendStart)) {
            // time to send the next symbol
            c = keyerMessage.message[currentSymbolIdx];
            isSending = true;

            if(c == '/') {
                // Pause between words.
                // We already waited for ditTimeMs, thus -1.
                // Act as if we are sending another symbol,
                // although keyDown() is not called.
                sendFinish = now + ditTimeMs*(7-1);
            } else if(c == ' ') {
                // Pause between letters.
                // We already waited for ditTimeMs, thus -1.
                // Act as if we are sending another symbol,
                // although keyDown() is not called.
                sendFinish = now + ditTimeMs*(3-1);
            } else if(c == '-') {
                keyDown();
                sendFinish = now + ditTimeMs*3;
            } else { // c == '.'
                keyDown();
                sendFinish = now + ditTimeMs;
            }
        }

        if(isSending && (now >= sendFinish)) {
            // another symbol was sent
            keyUp();
            symbolsLeft--;
            currentSymbolIdx++;
            isSending = false;

            // `ditTimeMs` is a pause between dits and dahs
            sendStart = now + ditTimeMs;
        }

        if(anyButtonPressed()) {
            break;
        }

        updateKeyerSpeed(false);
        HAL_Delay(5);
    }

    keyUp();
}

void recordMessage() {
    iambicKeyerLogReset();

    for(;;) {
        bool ditPressed = buttonDitPressed();
        bool dahPressed = buttonDahPressed();

        // use iambic keyer even in straight key mode
        processIambicKeyerLogic(ditPressed, dahPressed);

        if(buttonRecPressed()) {
            // SAVE confirmed
            keyerMessage.length = iambicKeyerLogSize();
            memcpy(keyerMessage.message, iambicKeyerLog, keyerMessage.length);
            
            // truncate the pauses in the end of the message
            while(keyerMessage.length > 0) {
                if(keyerMessage.message[keyerMessage.length-1] == ' ') {
                    keyerMessage.length--;
                    continue;
                }

                if(keyerMessage.message[keyerMessage.length-1] == '/') {
                    keyerMessage.length--;
                    continue;
                }

                break;
            }

            saveKeyerMessage();
            break;
        } else if(buttonSendPressed() || buttonPlayPressed()) {
            // SAVE aborted
            break;
        }

        updateKeyerSpeed(false);
        HAL_Delay(5);
    }
}

void displaySpeed() {
    if(speedWPM < 10) {
        // straight key mode
        displayDig1('-');
        displayDig2('-');
        return;
    }

    displayDig1((char)(speedWPM / 10) + '0');
    displayDig2((char)(speedWPM % 10) + '0');
}

uint8_t voltageToSpeed(double v) {
    const double v_max = 3.2;
    const double v_min = 0.1;
    if(v >= v_max) {
        return 30;
    }

    if(v <= v_min) {
        return 0; // straight key mode
    }

    return 10 + 20*(v - v_min)/(v_max - v_min);
}

void updateKeyerSpeed(bool display) {
    static double prev_voltage = 0.0;
    double speed_voltage;
    uint32_t target_speed;

    HAL_ADC_PollForConversion(&hadc1, HAL_MAX_DELAY);
    speed_voltage = ((double)HAL_ADC_GetValue(&hadc1))*3.3/4096.0;

    // histeresis
    if(fabs(speed_voltage - prev_voltage) < 0.05)
        return;

    target_speed = voltageToSpeed(speed_voltage);

    if(target_speed == speedWPM)
        return;

    prev_voltage = speed_voltage;
    speedWPM = target_speed;
    straightKey = (speedWPM < 10);
    if(straightKey) {
        // as for 10 WPM
        ditTimeMs = 60*1000/(50*10);
    } else {
        ditTimeMs = 60*1000/(50*speedWPM);
    }

    if(display) {
        displaySpeed();
    }
}

void init(void) {
    HAL_ADC_Start(&hadc1);
    HAL_TIM_PWM_Start(&htim1, TIM_CHANNEL_1);
    keyUp();

    loadKeyerMessage();
    displaySpeed();
}

void loop(void) {
    bool ditPressed = buttonDitPressed();
    bool dahPressed = buttonDahPressed();

    if(straightKey) {
        processStraightKeyerLogic(ditPressed);
    } else {
        processIambicKeyerLogic(ditPressed, dahPressed);
    }

    updateKeyerSpeed(true);

    if(buttonSendPressed()) {
        displayDig1('S');
        displayDig2(' ');

        playbackSavedMessage();

        displaySpeed();
    }

    if(buttonPlayPressed()) {
        displayDig1('P');
        displayDig2(' ');

        allowTransmit = false; // playback only
        playbackSavedMessage();
        allowTransmit = true;

        displaySpeed();
    }

    if(buttonRecPressed()) {
        displayDig1('R');
        displayDig2(' ');

        allowTransmit = false; // playback only
        recordMessage();
        allowTransmit = true;

        displaySpeed();
    }

    HAL_Delay(5);
}

/* USER CODE END 0 */

/**
  * @brief  The application entry point.
  * @retval int
  */
int main(void)
{
  /* USER CODE BEGIN 1 */

  /* USER CODE END 1 */

  /* MCU Configuration--------------------------------------------------------*/

  /* Reset of all peripherals, Initializes the Flash interface and the Systick. */
  HAL_Init();

  /* USER CODE BEGIN Init */

  /* USER CODE END Init */

  /* Configure the system clock */
  SystemClock_Config();

  /* USER CODE BEGIN SysInit */

  /* USER CODE END SysInit */

  /* Initialize all configured peripherals */
  MX_GPIO_Init();
  MX_I2C1_Init();
  MX_ADC1_Init();
  MX_TIM1_Init();
  /* USER CODE BEGIN 2 */
  init();
  /* USER CODE END 2 */

  /* Infinite loop */
  /* USER CODE BEGIN WHILE */
  while (1)
  {
    loop();
    /* USER CODE END WHILE */

    /* USER CODE BEGIN 3 */

  }
  /* USER CODE END 3 */
}

/**
  * @brief System Clock Configuration
  * @retval None
  */
void SystemClock_Config(void)
{
  RCC_OscInitTypeDef RCC_OscInitStruct = {0};
  RCC_ClkInitTypeDef RCC_ClkInitStruct = {0};
  RCC_PeriphCLKInitTypeDef PeriphClkInit = {0};

  /**Initializes the CPU, AHB and APB busses clocks 
  */
  RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSI;
  RCC_OscInitStruct.HSIState = RCC_HSI_ON;
  RCC_OscInitStruct.HSICalibrationValue = RCC_HSICALIBRATION_DEFAULT;
  RCC_OscInitStruct.PLL.PLLState = RCC_PLL_NONE;
  if (HAL_RCC_OscConfig(&RCC_OscInitStruct) != HAL_OK)
  {
    Error_Handler();
  }
  /**Initializes the CPU, AHB and APB busses clocks 
  */
  RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_HCLK|RCC_CLOCKTYPE_SYSCLK
                              |RCC_CLOCKTYPE_PCLK1|RCC_CLOCKTYPE_PCLK2;
  RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_HSI;
  RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
  RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV1;
  RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV1;

  if (HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_0) != HAL_OK)
  {
    Error_Handler();
  }
  PeriphClkInit.PeriphClockSelection = RCC_PERIPHCLK_ADC;
  PeriphClkInit.AdcClockSelection = RCC_ADCPCLK2_DIV2;
  if (HAL_RCCEx_PeriphCLKConfig(&PeriphClkInit) != HAL_OK)
  {
    Error_Handler();
  }
}

/**
  * @brief ADC1 Initialization Function
  * @param None
  * @retval None
  */
static void MX_ADC1_Init(void)
{

  /* USER CODE BEGIN ADC1_Init 0 */

  /* USER CODE END ADC1_Init 0 */

  ADC_ChannelConfTypeDef sConfig = {0};

  /* USER CODE BEGIN ADC1_Init 1 */

  /* USER CODE END ADC1_Init 1 */
  /**Common config 
  */
  hadc1.Instance = ADC1;
  hadc1.Init.ScanConvMode = ADC_SCAN_DISABLE;
  hadc1.Init.ContinuousConvMode = ENABLE;
  hadc1.Init.DiscontinuousConvMode = DISABLE;
  hadc1.Init.ExternalTrigConv = ADC_SOFTWARE_START;
  hadc1.Init.DataAlign = ADC_DATAALIGN_RIGHT;
  hadc1.Init.NbrOfConversion = 1;
  if (HAL_ADC_Init(&hadc1) != HAL_OK)
  {
    Error_Handler();
  }
  /**Configure Regular Channel 
  */
  sConfig.Channel = ADC_CHANNEL_4;
  sConfig.Rank = ADC_REGULAR_RANK_1;
  sConfig.SamplingTime = ADC_SAMPLETIME_1CYCLE_5;
  if (HAL_ADC_ConfigChannel(&hadc1, &sConfig) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN ADC1_Init 2 */

  /* USER CODE END ADC1_Init 2 */

}

/**
  * @brief I2C1 Initialization Function
  * @param None
  * @retval None
  */
static void MX_I2C1_Init(void)
{

  /* USER CODE BEGIN I2C1_Init 0 */

  /* USER CODE END I2C1_Init 0 */

  /* USER CODE BEGIN I2C1_Init 1 */

  /* USER CODE END I2C1_Init 1 */
  hi2c1.Instance = I2C1;
  hi2c1.Init.ClockSpeed = 100000;
  hi2c1.Init.DutyCycle = I2C_DUTYCYCLE_2;
  hi2c1.Init.OwnAddress1 = 0;
  hi2c1.Init.AddressingMode = I2C_ADDRESSINGMODE_7BIT;
  hi2c1.Init.DualAddressMode = I2C_DUALADDRESS_DISABLE;
  hi2c1.Init.OwnAddress2 = 0;
  hi2c1.Init.GeneralCallMode = I2C_GENERALCALL_DISABLE;
  hi2c1.Init.NoStretchMode = I2C_NOSTRETCH_DISABLE;
  if (HAL_I2C_Init(&hi2c1) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN I2C1_Init 2 */

  /* USER CODE END I2C1_Init 2 */

}

/**
  * @brief TIM1 Initialization Function
  * @param None
  * @retval None
  */
static void MX_TIM1_Init(void)
{

  /* USER CODE BEGIN TIM1_Init 0 */

  /* USER CODE END TIM1_Init 0 */

  TIM_ClockConfigTypeDef sClockSourceConfig = {0};
  TIM_MasterConfigTypeDef sMasterConfig = {0};
  TIM_OC_InitTypeDef sConfigOC = {0};
  TIM_BreakDeadTimeConfigTypeDef sBreakDeadTimeConfig = {0};

  /* USER CODE BEGIN TIM1_Init 1 */

  /* USER CODE END TIM1_Init 1 */
  htim1.Instance = TIM1;
  htim1.Init.Prescaler = 44;
  htim1.Init.CounterMode = TIM_COUNTERMODE_UP;
  htim1.Init.Period = 256;
  htim1.Init.ClockDivision = TIM_CLOCKDIVISION_DIV1;
  htim1.Init.RepetitionCounter = 0;
  htim1.Init.AutoReloadPreload = TIM_AUTORELOAD_PRELOAD_DISABLE;
  if (HAL_TIM_Base_Init(&htim1) != HAL_OK)
  {
    Error_Handler();
  }
  sClockSourceConfig.ClockSource = TIM_CLOCKSOURCE_INTERNAL;
  if (HAL_TIM_ConfigClockSource(&htim1, &sClockSourceConfig) != HAL_OK)
  {
    Error_Handler();
  }
  if (HAL_TIM_PWM_Init(&htim1) != HAL_OK)
  {
    Error_Handler();
  }
  sMasterConfig.MasterOutputTrigger = TIM_TRGO_RESET;
  sMasterConfig.MasterSlaveMode = TIM_MASTERSLAVEMODE_DISABLE;
  if (HAL_TIMEx_MasterConfigSynchronization(&htim1, &sMasterConfig) != HAL_OK)
  {
    Error_Handler();
  }
  sConfigOC.OCMode = TIM_OCMODE_PWM1;
  sConfigOC.Pulse = 0;
  sConfigOC.OCPolarity = TIM_OCPOLARITY_HIGH;
  sConfigOC.OCNPolarity = TIM_OCNPOLARITY_HIGH;
  sConfigOC.OCFastMode = TIM_OCFAST_DISABLE;
  sConfigOC.OCIdleState = TIM_OCIDLESTATE_RESET;
  sConfigOC.OCNIdleState = TIM_OCNIDLESTATE_RESET;
  if (HAL_TIM_PWM_ConfigChannel(&htim1, &sConfigOC, TIM_CHANNEL_1) != HAL_OK)
  {
    Error_Handler();
  }
  sBreakDeadTimeConfig.OffStateRunMode = TIM_OSSR_DISABLE;
  sBreakDeadTimeConfig.OffStateIDLEMode = TIM_OSSI_DISABLE;
  sBreakDeadTimeConfig.LockLevel = TIM_LOCKLEVEL_OFF;
  sBreakDeadTimeConfig.DeadTime = 0;
  sBreakDeadTimeConfig.BreakState = TIM_BREAK_DISABLE;
  sBreakDeadTimeConfig.BreakPolarity = TIM_BREAKPOLARITY_HIGH;
  sBreakDeadTimeConfig.AutomaticOutput = TIM_AUTOMATICOUTPUT_DISABLE;
  if (HAL_TIMEx_ConfigBreakDeadTime(&htim1, &sBreakDeadTimeConfig) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN TIM1_Init 2 */

  /* USER CODE END TIM1_Init 2 */
  HAL_TIM_MspPostInit(&htim1);

}

/**
  * @brief GPIO Initialization Function
  * @param None
  * @retval None
  */
static void MX_GPIO_Init(void)
{
  GPIO_InitTypeDef GPIO_InitStruct = {0};

  /* GPIO Ports Clock Enable */
  __HAL_RCC_GPIOC_CLK_ENABLE();
  __HAL_RCC_GPIOA_CLK_ENABLE();
  __HAL_RCC_GPIOB_CLK_ENABLE();

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(GPIOC, GPIO_PIN_13|GPIO_PIN_14|GPIO_PIN_15, GPIO_PIN_RESET);

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(GPIOA, GPIO_PIN_0|GPIO_PIN_1|GPIO_PIN_2|GPIO_PIN_3 
                          |GPIO_PIN_5|GPIO_PIN_6|GPIO_PIN_7, GPIO_PIN_RESET);

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(GPIOB, GPIO_PIN_0|GPIO_PIN_1|GPIO_PIN_10|GPIO_PIN_11 
                          |GPIO_PIN_12, GPIO_PIN_RESET);

  /*Configure GPIO pins : PC13 PC14 PC15 */
  GPIO_InitStruct.Pin = GPIO_PIN_13|GPIO_PIN_14|GPIO_PIN_15;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init(GPIOC, &GPIO_InitStruct);

  /*Configure GPIO pins : PA0 PA1 PA2 PA3 
                           PA5 PA6 PA7 */
  GPIO_InitStruct.Pin = GPIO_PIN_0|GPIO_PIN_1|GPIO_PIN_2|GPIO_PIN_3 
                          |GPIO_PIN_5|GPIO_PIN_6|GPIO_PIN_7;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);

  /*Configure GPIO pins : PB0 PB1 PB10 PB11 
                           PB12 */
  GPIO_InitStruct.Pin = GPIO_PIN_0|GPIO_PIN_1|GPIO_PIN_10|GPIO_PIN_11 
                          |GPIO_PIN_12;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init(GPIOB, &GPIO_InitStruct);

  /*Configure GPIO pins : PB13 PB14 */
  GPIO_InitStruct.Pin = GPIO_PIN_13|GPIO_PIN_14;
  GPIO_InitStruct.Mode = GPIO_MODE_INPUT;
  GPIO_InitStruct.Pull = GPIO_PULLUP;
  HAL_GPIO_Init(GPIOB, &GPIO_InitStruct);

  /*Configure GPIO pins : PA11 PA12 PA15 */
  GPIO_InitStruct.Pin = GPIO_PIN_11|GPIO_PIN_12|GPIO_PIN_15;
  GPIO_InitStruct.Mode = GPIO_MODE_INPUT;
  GPIO_InitStruct.Pull = GPIO_PULLUP;
  HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);

}

/* USER CODE BEGIN 4 */

/* USER CODE END 4 */

/**
  * @brief  This function is executed in case of error occurrence.
  * @retval None
  */
void Error_Handler(void)
{
  /* USER CODE BEGIN Error_Handler_Debug */
  /* User can add his own implementation to report the HAL error return state */
  while(1) 
  {
  }
  /* USER CODE END Error_Handler_Debug */
}

#ifdef  USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t *file, uint32_t line)
{ 
  /* USER CODE BEGIN 6 */
  /* User can add his own implementation to report the file name and line number,
    ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
  /* USER CODE END 6 */
}
#endif /* USE_FULL_ASSERT */

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
